﻿define(['underscore', 'moment', 'Autolinker'], function(_, moment, Autolinker) {

    var autolinkerInstance =  new Autolinker({
        className: 'tb-link',
        urls: true,
        email: false,
        twitter: false,
        stripPrefix: true,
        truncate: 30
    });

    return {
        serviceDateFormat: 'YYYY-MM-DD',
        serviceDateTimeFormat: 'YYYY-MM-DDTHH:mm:ss\\Z',

        humanDateFormat: 'DD.MM.YYYY',
        humanDateTimeFormat: 'DD.MM.YYYY HH:mm',
        humanDateTimeActivityFormat: 'D MMM HH:mm',
        humanDateTimeVerbalFormat: 'DD MMM YYYY HH:mm',
        humanTimeFormat: 'HH:mm',

        fileNameDateTimeFormat: 'YYYYMMDD-HHmmss',

        todayAsString: function() {
            return moment().format(this.serviceDateFormat);
        },

        nowAsString: function() {
            return moment().format(this.serviceDateTimeFormat);
        },

        nowAsFileNameString: function() {
            return moment().format(this.fileNameDateTimeFormat);
        },

        serviceDateTimeToHuman: function(dateTimeInServiceFormat) {
            return dateTimeInServiceFormat ? moment(dateTimeInServiceFormat).format(this.humanDateTimeFormat) : '';
        },

        serviceDateTimeToHumanVerbal: function(dateTimeInServiceFormat) {
            return dateTimeInServiceFormat ? moment(dateTimeInServiceFormat).format(this.humanDateTimeVerbalFormat) : '';
        },

        serviceDateToHuman: function(dateInServiceFormat) {
            return dateInServiceFormat ? moment(dateInServiceFormat).format(this.humanDateFormat) : '';
        },

        humanDateToService: function(dateInHumanFormat) {
            return dateInHumanFormat ? moment(dateInHumanFormat, this.humanDateFormat).format(this.serviceDateFormat) : '';
        },

        serviceDateToServiceDateTime: function(dateInServiceFormat) {
            return dateInServiceFormat ? moment(dateInServiceFormat).format(this.serviceDateTimeFormat) : '';
        },

        endOfDay: function(dateInServiceFormat) {
            return dateInServiceFormat ? dateInServiceFormat + 'T23:59:59Z' : '';
        },

        startOfDay: function(dateInServiceFormat) {
            return dateInServiceFormat ? dateInServiceFormat + 'T00:00:00Z' : '';
        },

        caption: function(ref) {
            return ref ? _.escape(ref.caption) : '';
        },

        captions: function(refs, delimiter) {
            delimiter = delimiter || ', ';
            if (refs instanceof Array) {
                return _.map(refs, function(ref) { return ref.caption; }).join(delimiter);
            } else {
                return this.caption(refs);
            }
        },

        formatTimeRangeForEvent: function(eventModel, forceShowDate) {
            forceShowDate = forceShowDate || false;
            var startMoment = this.timeOnToMoment(eventModel.get('startOn'));
            var dueMoment = this.timeOnToMoment(eventModel.get('dueOn'));
            if (startMoment.isSame(dueMoment, 'day')) {
                if (forceShowDate) {
                    return startMoment.format(this.humanDateFormat) + '  ' +
                        startMoment.format(this.humanTimeFormat) + ' - ' +
                        dueMoment.format(this.humanTimeFormat);
                } else {
                    return startMoment.format(this.humanTimeFormat) + ' - ' +
                        dueMoment.format(this.humanTimeFormat);
                }

            } else {
                return startMoment.format(this.humanDateTimeFormat) + ' - ' +
                    dueMoment.format(this.humanDateTimeFormat);
            }
        },

        formatTimeOnForTask: function(timeOn, currentYear) {
            if (timeOn) {
                if (timeOn.date) {
                    var m = moment(timeOn.date);
                    return currentYear == m.year() ? m.format('D MMM') : m.format('D MMM YYYY');
                } else if (timeOn.dateTime) {
                    var m = moment(timeOn.dateTime);
                    return currentYear == m.year() ? m.format('D MMM, HH:mm') : m.format('D MMM YYYY, HH:mm');
                }
            }
            return '';
        },

        timeOnToGivenFormat: function(timeOn, dateFormat, dateTimeFormat, options) {
            if (timeOn) {
                if (timeOn.date) {
                    return moment(timeOn.date).format(dateFormat);
                } else if (timeOn.dateTime) {
                    var m = moment(timeOn.dateTime);
                    return (options && options.truncateTime) ? m.format(dateFormat) : m.format(dateTimeFormat);
                }
            }
            return '';
        },

        timeOnToHuman: function(timeOn, options) {
            return this.timeOnToGivenFormat(timeOn, this.humanDateFormat, this.humanDateTimeFormat, options);
        },

        timeOnToService: function(timeOn, options) {
            return this.timeOnToGivenFormat(timeOn, this.serviceDateFormat, this.serviceDateTimeFormat, options);
        },

        timeOnToMoment: function(timeOn) {
            if (timeOn) {
                if (timeOn.date) {
                    return moment(timeOn.date);
                } else if (timeOn.dateTime) {
                    return moment(timeOn.dateTime);
                }
            }
            return undefined;
        },

        isServiceDate: function(s) {
            return moment(s, this.serviceDateFormat, true).isValid();
        },

        isServiceDateTime: function(s) {
            return moment(s, this.serviceDateTimeFormat, true).isValid();
        },

        humanizeDateTime: function(dateTimeInServiceFormat) {
            // TODO correct this, use 'Yesterday, 12:03' notation
            return moment(dateTimeInServiceFormat).format(this.humanDateTimeFormat);
        },

        humanizeActivityDateTime: function(dateTimeInServiceFormat) {
            return moment(dateTimeInServiceFormat).format(this.humanDateTimeActivityFormat);
        },

        futureMoment: function() {
            return moment('2202-07-06');
        },

        dateTime: function(date, time) {
            if (!date) {
                return undefined;
            }
            time = time || '00:00:00';
            if (/[\d]{2}:[\d]{2}/.test(time)) {
                time = time + ':00';
            } else if (/[\d]{1}:[\d]{2}/.test(time)) {
                time = '0' + time + ':00';
            }
            return date + 'T' + time + 'Z';
        },

        cleanTags: function(s, processNewLines) {
            if (!s) return '';
            while (true) {
                var tagStartIndex = s.indexOf('<');
                if (tagStartIndex < 0) {
                    break;
                }
                var tagEndIndex = s.indexOf('>', tagStartIndex) + 1;
                if (tagEndIndex <= 0) {
                    tagEndIndex = s.length;
                }
                var tag = s.substring(tagStartIndex, tagEndIndex);
                s = s.replace(tag, '');
            }
            if (processNewLines) {
                return this.processNewLines(s);
            }
            return s;
        },

        processNewLines: function(s) {
            if (!s) return '';
            return s = s.replace(/[\r\n]+/g, '<br>');
        },

        autoLink: function(s) {
            if (!s) return '';
            return autolinkerInstance.link(s);
        }

    }

});