﻿/**
 * @class CalendarView
 */
define([
    'underscore',
    'jquery',
    'views/BaseToolView',
    'text!views/calendar/CalendarView.html',
    'textOriginal!views/calendar/CalendarView_calendarTemplate.html',
    'settings',
    'srv',
    'clndr',
    'utils/navigationUtil',
    'views/common/BaseLowerToolbar',
    'views/filterSupport',
    'views/calendar/CalendarFiltersPopupView',
    'moment',
    'utils/cache/allUsersCache',
    'utils/cache/currentUserCache'
], function (_, $, BaseToolView, contentTemplate, calendarTemplate, settings, srv, clndr, navigationUtil, BaseLowerToolbar, filterSupport, CalendarFiltersPopupView, moment, allUsersCache, currentUserCache) {
    return BaseToolView.extend({

        contentTemplate: contentTemplate,

        headerText: 'Календарь',

        filterPopupClass: CalendarFiltersPopupView,

        cardEntityType: 'calendarEvent',

        events: _.extend({
            'click .js-previousMonth': 'onPreviousMonth',
            'click .js-nextMonth': 'onNextMonth',
            'click .js-dayLink': 'onDayLinkClicked',
            'swipeRight .js-calendarContainer': 'onPreviousMonth',
            'swipeLeft .js-calendarContainer': 'onNextMonth'
        }, BaseToolView.prototype.events),

        rightButton: {
            icon: 'plus',
            handler: 'onAddClicked'
        },

        refData: _.extend({
            allUsers: allUsersCache.refDataLoader,
            currentUser: currentUserCache.refDataLoader
        }, BaseToolView.prototype.refData),


        startDate: null,

        calendarData: null,
        _calendarDataLoaded: null,

        initialize: function (options) {
            this.include(filterSupport);
            BaseToolView.prototype.initialize.apply(this, arguments);

            if (options.yearAndMonth) {
                this.startDate = moment(options.yearAndMonth, 'YYYY-MM').startOf('month');
            } else {
                this.startDate = moment().startOf('month');
            }

            this.restoreFilters();
            this._initializeLowerToolbar();
        },

        _initializeLowerToolbar: function () {
            var that = this;
            if (!this.lowerToolbarClass) {
                this.lowerToolbarClass = BaseLowerToolbar.extend({
                    items: [
                        {
                            builtin: 'refresh'
                        },
                        {
                            builtin: 'filter'
                        }
                    ]
                });
            }
        },

        fetchParams: function () {
            var params = {};
            var dateTimeFromParam = moment(this.startDate).startOf('month').subtract('days', 7).format(this.formatter.serviceDateFormat);
            var dateTimeToParam = moment(this.startDate).endOf('month').add('days', 7).format(this.formatter.serviceDateFormat);
            _.extend(params, {
                dateFrom: dateTimeFromParam,
                dateTo: dateTimeToParam,
             });
            if (this.filters) {
                this.filters.dataType && (params.dataType = this.filters.dataType);
                this.filters.assigneeOrMember && (params.assigneeOrMember = this.filters.assigneeOrMember);
            }
            return params;
        },

        fetchUrl: function () {
            // TODO must be areaId, workspaceId here as attributes (???)
            return srv.makeUrlWithLocationContext({
                area: this.options.areaId,
                workspace: this.options.workspaceId
            }, 'calendar');

        },

        updateData: function (updateOptions) {
            var that = this;
            if (!this.startDate) {
                this.startDate = moment().startOf('month');
            }
            this._mainDataLoaded = false;
            this.calendarData = null;

            var url = this.fetchUrl();
            var urlParams = this.fetchParams();
            srv.fetch(_.defaults({
                urlParams: urlParams,
                url: url,
                success: function (result) {
                    that.calendarData = result;
                    settings.ajaxDebugging && that.log.debug('CalendarView: got calendar data', {
                        url: url,
                        urlParams: urlParams,
                        taskCount: (that.calendarData && that.calendarData.tasks) ? that.calendarData.tasks.length : 'No tasks',
                        eventCount: (that.calendarData && that.calendarData.calendarEvents) ? that.calendarData.calendarEvents.length : 'No events'
                    });
                    that.onDataUpdated();
                }
            }, updateOptions));
        },

        render: function () {
            var that = this;
            BaseToolView.prototype.render.apply(this, arguments);
            this.$('.js-calendarContainer').clndr({
                template: calendarTemplate,
                startWithMonth: that.startDate,
                daysOfTheWeek: ['пн', 'вт', 'ср', 'чт', 'пт', 'сб', 'вс'],
                //clickEvents: {
                //    onMonthChange: function (month) {
                //    }
                //}
            });
        },

        renderData: function () {
            var that = this;
            this.renderLocationContextCaption();
            setTimeout(function () {
                that._renderCalendarItems();
            }, 100);

        },

        _renderCalendarItems: function ()
        {
        	var that = this;
        	var data = {};

        	function counts(formattedDateValue)
        	{
        		var counts = data[formattedDateValue];
        		if (!counts)
        		{
        			counts = {
        				taskCount: 0,
        				eventCount: 0
        			};
        			data[formattedDateValue] = counts;
        		}
        		return counts;
        	}

        	if (!_.isEmpty(this.calendarData))
        	{
        		function iterator(item, countAttributeName)
        		{
        			var startDate = that.formatter.timeOnToService(item.startOn, { truncateTime: true });
        			var dueDate = that.formatter.timeOnToService(item.dueOn, { truncateTime: true });
        			if (countAttributeName == 'taskCount')
        			{
        				if (!startDate && dueDate)
        				{
        					counts(dueDate)[countAttributeName]++;
        				}
        				else
        				{
        					if (startDate !== dueDate)
        					{
        						var m1 = moment(startDate, that.formatter.serviceDateFormat);
        						var m2 = moment(dueDate, that.formatter.serviceDateFormat);
        						for (var m = m1; m.isBefore(m2) ; m.add('days', 1))
        						{
        							counts(m.format(that.formatter.serviceDateFormat))[countAttributeName]++;
        						}
        						counts(dueDate)[countAttributeName]++;
        					}
        					else
        						counts(dueDate)[countAttributeName]++;
        				}
        			}
        			if (countAttributeName == 'eventCount')
        			{

        				if (startDate !== dueDate)
        				{
        					var m1 = moment(startDate, that.formatter.serviceDateFormat);
        					var m2 = moment(dueDate, that.formatter.serviceDateFormat);
        					for (var m = m1; m.isBefore(m2) ; m.add('days', 1))
        					{
        						counts(m.format(that.formatter.serviceDateFormat))[countAttributeName]++;
        					}
        					counts(dueDate)[countAttributeName]++;
        				}
        				else
        					counts(dueDate)[countAttributeName]++;
        			}
        		}

        		_.each(this.calendarData.tasks, function (item)
        		{
        			iterator(item, 'taskCount');
        		});
        		_.each(this.calendarData.calendarEvents, function (item)
        		{
        			iterator(item, 'eventCount');
        		});
        	}
        	this.$('.js-taskCount').add('.js-eventCount').remove();
        	_.each(data, function (counts, date)
        	{
        		var $td = that.$('.calendar-day-' + date);
        		var hasContent = false;
        		if (counts.taskCount > 0)
        		{
        			$('<span></span>').text(counts.taskCount).addClass('tb-taskCount js-taskCount').appendTo($td);
        			hasContent = true;
        		}
        		if (counts.eventCount)
        		{
        			$('<span></span>').text(counts.eventCount).addClass('tb-eventCount js-eventCount').appendTo($td);
        			hasContent = true;
        		}
        		if (hasContent)
        		{
        			$td.addClass('js-hasContent');
        		}
        	});
        },

        currentMonthAndYearAsString: function () {
            return this.startDate.format('MMM YYYY');
        },

        nextMonthAsString: function () {
            return this.startDate.clone().add('months', 1).format('MMMM');
        },

        previousMonthAsString: function () {
            return this.startDate.clone().subtract('months', 1).format('MMMM');
        },

        onPreviousMonth: function (evt) {
            /*this.startDate.subtract('months', 1);
            this.render();*/
        	//evt.preventDefault();
        	var prevMonth = moment(this.startDate).subtract('months', 1).format('YYYY-MM');
        	var args = {
        		yearAndMonth: prevMonth
        	};
        	var locationContext = this.locationContext();
        	if (!_.isEmpty(locationContext))
        	{
        		args.area = locationContext.areaId;
        		args.workspace = locationContext.workspaceId;
        	}
        	this.router.navigate(this.urlBuilder.generic('calendar', args), { trigger: true, replace: false });
        },

        onNextMonth: function (evt) {
            /*this.startDate.add('months', 1);
            this.render();*/
        	//evt.preventDefault();
        	var nextMonth = moment(this.startDate).add('months', 1).format('YYYY-MM');
        	var args = {
        		yearAndMonth: nextMonth
        	};
        	var locationContext = this.locationContext();
        	if (!_.isEmpty(locationContext))
        	{
        		args.area = locationContext.areaId;
        		args.workspace = locationContext.workspaceId;
        	}
        	this.router.navigate(this.urlBuilder.generic('calendar', args), { trigger: true, replace: false });
        },

        onDayLinkClicked: function (evt)
        {
        	//evt.preventDefault();
        	var $td = $(evt.currentTarget).closest('td');
        	//if ($td.hasClass('js-hasContent'))
        	//{
        		var classes = $td.attr('class').split(/\s+/);
        		var dateClass = _.find(classes, function (cssClass)
        		{
        			return cssClass && cssClass.indexOf('calendar-day-') === 0;
        		});
        		if (dateClass)
        		{
        			var date = dateClass.substring(13); // 13 is the length of 'calendar-day-' string
        			var args = {
        				date: date
        			};
        			var locationContext = this.locationContext();
        			if (!_.isEmpty(locationContext))
        			{
        				args.area = locationContext.areaId;
        				args.workspace = locationContext.workspaceId;
        			}
        			_.extend(args, this.filters);
        			this.router.navigate(this.urlBuilder.generic('calendar', args), { trigger: true, replace: false });
        		}
        	//}
        },

        onAddClicked: function (evt)
        {
        	//evt.preventDefault();
        	var locationContext = this.locationContext();
        	navigationUtil.showEntityCreationMenu(this.router, ['task', 'calendarEvent'], locationContext);
        }

    });
});